import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Random;
import javax.swing.*;

/**
 * This project will serve as your primary portfolio of software achievements
 * for ICS3U and, hopefully, ICS4U that is to be mounted on your home page. In
 * this way, it will serve to support for your Resume and Curriculum Vitae that
 * you have need of in the years ahead.
 */
public class Framework extends JApplet implements ActionListener {
	// The program's base dimensions.
	private static Dimension appDimen = new Dimension(600, 600);
	// The singular panel onto which will be drawn the individual 'Drawable'
	// activities
	private static Content content;
	private static JFrame frame;
	private static String title = "Your Name's Project Framework";
	private static JMenu menu;

	public static void main(String[] args) {
		frame = new JFrame();

		frame.setTitle(title);
		// Terminate process when window is closed(Application only)
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		// Center, if desired (Application only)
		GraphicsConfiguration gc = frame.getGraphicsConfiguration();
		Rectangle rect = gc.getBounds();
		frame.setLocation((rect.width - appDimen.width) / 2,
				(rect.height - appDimen.height) / 2);

		// create alternative applet interpretation
		JApplet applet = new Framework();
		applet.init();
		frame.getContentPane().add(applet);

		// final preparations for presentation
		frame.pack();
		frame.setResizable(false);
		frame.setVisible(true);
		applet.start();

	}

	/**
	 * Entry point for browsers in launching Applet.
	 */
	public void init() {
		content = new Content();
		getContentPane().add(content);

		JMenuBar mb = new JMenuBar();
		setJMenuBar(mb);

		menu = new JMenu("Searching");
		mb.add(menu);
		addMenuItem("Sequential");
		menu.addSeparator();
		addMenuItem("Binary");

		menu = new JMenu("Life");
		mb.add(menu);
		addMenuItem("Random");
		
		menu = new JMenu("About");
		mb.add(menu);
		addMenuItem("Welcome");

	}

	public void start() {
		content.createNewImage();
		content.repaint();
	}

	private void addMenuItem(String label) {
		JMenuItem mi = new JMenuItem(label);
		mi.setActionCommand(label);
		menu.add(mi);
		mi.addActionListener(this);
	}

	/**
	 * Sole method required by ActionListener interface. Determines which menu
	 * item was selected by examining the ActionEvent passed by the Listener.
	 */
	public void actionPerformed(ActionEvent ev) {
		String command = ev.getActionCommand();
		// Comment out the statement below when running as an Applet
		// frame.setTitle(title + ": " + command);
		if ("Sequential".equals(command)) {
			content.current = new SequentialSearch(30);
			content.startThread(200);
		} else if ("Binary".equals(command)) {
			content.current = new BinarySearch(30);
			content.startThread(200);
		} else if ("Welcome".equals(command)) {
			content.current = new Welcome(appDimen);
			content.createNewImage();
			repaint();
		}

	}

	public class Content extends JPanel implements Runnable {
		// The temporary buffered image used for drawing off screen...
		private BufferedImage img;
		// The reference to img's drawing canvas...
		private Graphics2D drawImg;
		// The active drawable curve...
		private Drawable current;
		// SPEED controls the default delay of the utility Thread
		int sleep = 500;
		int defaultSleep = sleep;
		public Thread thread;

		/**
		 * Constructor initializes the reference to the default Drawable image
		 * (current) to the Welcome Screen and a few other references and
		 * variable
		 */
		public Content() {
			current = new Welcome(appDimen);
			setPreferredSize(appDimen);
			img = new BufferedImage(appDimen.width, appDimen.height,
					BufferedImage.TYPE_INT_RGB);
			drawImg = img.createGraphics();
			// In case any of the projects ahead require mouse support
			MouseListener mouseListener = new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {
					current.setMouseEvent(e);
					createNewImage();
					repaint();
				}
			};
			addMouseListener(mouseListener);
			this.setVisible(true);
			requestFocus();

		}

		public void startThread(int sleep) {
			thread = null;
			this.sleep = sleep;
			thread = new Thread(this);
			thread.start();
		}

		public void stopThread() {
			thread = null;

		}

		/**
		 * Creates and sets up a new <code>BufferedImage</code> with
		 * <code>this component</code>'s size, and draws the current
		 * <code>Drawable</code> onto the <code>BufferedImage</code>.
		 */
		public void createNewImage() {
			try {
				img = new BufferedImage(getWidth(), getHeight(),
						BufferedImage.TYPE_INT_RGB);
				drawImg = img.createGraphics();
				drawImg.setBackground(current.getBackgroundColor());
				drawImg.clearRect(0, 0, getWidth(), getHeight());
				current.draw(img);
			} catch (IllegalArgumentException e) {
			}
		}

		/**
		 * Returns the main <code>BufferedImage</code> being used by the
		 * <code>paintComponent</code> method.
		 * 
		 * @return the <code>BufferedImage</code> currently being used as a
		 *         graphic.
		 */
		public BufferedImage getImage() {
			return img;
		}

		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			Graphics2D g2D = (Graphics2D) g;
			g2D.drawImage(getImage(), 0, 0, this);
		}

		public void setSleep(int s) {
			sleep = s;
		}

		public void run() {
			Thread thisThread = Thread.currentThread();
			while (thread == thisThread && !current.finished()) {
				createNewImage();
				repaint();
				try {
					Thread.sleep(sleep);
				} catch (InterruptedException e) {
				}
			}
		}

		public void saveImage() {
			current.saveImage();
		}

	}

	/**
 * 
 */
	public class SequentialSearch implements Drawable{

		public SequentialSearch(int n){
			
		}

		@Override
		public void draw(BufferedImage drawTo) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public Color getBackgroundColor() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public void setMouseEvent(MouseEvent e) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public boolean finished() {
			// TODO Auto-generated method stub
			return false;
		}

		@Override
		public void setCompleted() {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void saveImage() {
			// TODO Auto-generated method stub
			
		}
		
	}

	/**
	 * 
	 */
	public class BinarySearch implements Drawable{
		public BinarySearch(int n){
			
		}

		@Override
		public void draw(BufferedImage drawTo) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public Color getBackgroundColor() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public void setMouseEvent(MouseEvent e) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public boolean finished() {
			// TODO Auto-generated method stub
			return false;
		}

		@Override
		public void setCompleted() {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void saveImage() {
			// TODO Auto-generated method stub
			
		}

	}

	/**
	 * Initial implementation of the Drawable interface that will determine and
	 * display a number of rows of Pascal's Triangle. Declared private solely
	 * for the purpose of javadoc.
	 */
	public class Welcome implements Drawable {
		private Color bgColor, fgColor;
		boolean completed;
		int width, height;
		Dimension appDimen;

		public Welcome(Dimension appDimen) {
			width = appDimen.width;
			height = appDimen.height;
			bgColor = Color.white;
			fgColor = Color.black;
		}

		/**
		 * Called by respective Drawable's createNewGraphics' method. This is
		 * where the specific drawing takes place.
		 */

		public void draw(BufferedImage img) {
			Graphics2D g = (Graphics2D) img.createGraphics();
			g.setBackground(getBackgroundColor());
			g.setColor(fgColor);
			String str = "RSGC ACES";
			g.setFont(new Font("Arial", Font.BOLD, 24));
			FontMetrics fm = g.getFontMetrics();
			Rectangle2D area = fm.getStringBounds(str, g);
			g.drawString(str, (int) (width - area.getWidth()) / 2, height / 3);
			str = "Your Name's Project Framework";
			g.setFont(new Font("Arial", Font.BOLD, 36));
			fm = g.getFontMetrics();
			area = fm.getStringBounds(str, g);
			g.drawString(str, (int) (width - area.getWidth()) / 2,
					height / 3 + 64);
			str = "Portfolio of Software Projects";
			g.setFont(new Font("Arial", Font.BOLD, 16));
			fm = g.getFontMetrics();
			area = fm.getStringBounds(str, g);
			g.drawString(str, (int) (width - area.getWidth()) / 2,
					height / 3 + 120);

		}

		public Color getBackgroundColor() {
			return bgColor;
		}

		@Override
		public void setMouseEvent(MouseEvent e) {
			// TODO Auto-generated method stub

		}

		@Override
		public boolean finished() {
			return true;

		}

		@Override
		public void setCompleted() {
			completed = true;

		}

		@Override
		public void saveImage() {
			// TODO Auto-generated method stub

		}

	}

	/**
	 * Simple interface used for all Drawable classes. Every Drawable object
	 * requires a method to draw to a given <code>BufferedImage</code>, and a
	 * method returning the background color used.
	 * 
	 * @version 1.0
	 * @author iComery
	 */
	public interface Drawable {
		Color fgColor = Color.green;
		Color bgColor = Color.black;
		Color[] palette = { Color.red, Color.green, Color.blue };
		Random random = new Random();

		// boolean completed = false;
		/**
		 * Draws itself to the parameterized <code>BufferedImage</code>.
		 * 
		 * @param drawTo
		 *            a <code>BufferedImage</code> to draw to.
		 */
		public abstract void draw(BufferedImage drawTo);

		/**
		 * Merely returns the background color for <code>this Drawable</code>.
		 * 
		 * @return the <code>Color</code> of the background for
		 *         <code>this Drawable</code>.
		 */
		public abstract Color getBackgroundColor();

		/**
		 * Allows all implementers to react to mouse events passed from the
		 * content class.
		 * 
		 * @param e
		 *            The <code>MouseEvent</code> that the object is to respond
		 *            to.
		 */
		public abstract void setMouseEvent(MouseEvent e);

		/**
		 * Sets the status of the rendering to completed. Called by Content's
		 * stopThread method to interrupt and terminate the rendering thread.
		 */
		public abstract boolean finished();

		/**
		 * Sets the status of the rendering to completed. Called by Content's
		 * stopThread method to interrupt and terminate the rendering thread.
		 */
		public abstract void setCompleted();

		public abstract void saveImage();

	}

}
